#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#define WIDTH 1000
#define HEIGHT 500
#define NUKEEXPANDSPEED 20
#define STARTRADIUS 0
#define MAXRADIUS 20 
#define RAINDELAY 5000

struct nuke
	{
	int x,y;	// position
	int radius;	// current radius
	int rd;		// direction radius is expanding in, 1 or -1
	int tickerID;	// id of ticker function
	GtkWidget *darea;
	GSList *list;	// the list that I'm in
	int red,green,blue;
	};
typedef struct nuke nuke;

GtkWidget *createDrawWindow(int xsize,int ysize,int xoff,int yoff);
GtkWidget *createDrawArea(GtkWidget *window,GtkWidget *vbox,int xsize,int ysize);
GtkWidget *createVBox(GtkWidget *window);
gint expose_event (GtkWidget *widget, GdkEventExpose *event);
gint configure_event (GtkWidget *widget, GdkEventConfigure *event);
gint button_release_event(GtkWidget *widget, GdkEventButton *event);
void quit(void);
gint timeout_cb(gpointer *data);
void repaint(GtkWidget *w);
nuke *newNuke(void);
void destroyNuke(nuke *n);
gint expandNuke(gpointer *data);
gint motion_notify_event (GtkWidget *widget, GdkEventMotion *event);
void dropNuke(GSList *nukes,gint x,gint y,GtkWidget *darea,int r,int g,int b);
gint dropRain(gpointer *data);

GdkPixmap *pixmap=NULL;

int main(int argc, char *argv[])
{
GtkWidget *window,*darea,*vbox;
GSList *nukes;
GdkCursor *cursor;

nukes = g_slist_alloc();

gtk_init(&argc,&argv);

window = createDrawWindow(WIDTH,HEIGHT,20,20);
vbox = createVBox(window);
darea = createDrawArea(window,vbox,WIDTH,HEIGHT);

gtk_object_set_data(GTK_OBJECT(window),"darea",darea);
gtk_object_set_data(GTK_OBJECT(window),"nukes",nukes);

gtk_widget_show(window);

cursor = gdk_cursor_new(GDK_CROSS);
gdk_window_set_cursor(darea->window,cursor);

//rainID = gtk_timeout_add(RAINDELAY,(GtkFunction)dropRain,(gpointer)window);

gtk_main();

g_slist_free(nukes);
gdk_cursor_destroy(cursor);

return 0;
}

gint dropRain(gpointer *data)
{
GSList *nukes;
GtkWidget *darea;

darea = gtk_object_get_data(GTK_OBJECT(data),"darea");
nukes = gtk_object_get_data(GTK_OBJECT(data),"nukes");

dropNuke(nukes,rand()%WIDTH,rand()%HEIGHT,darea,1,0,0);

return TRUE;
}

nuke *newNuke(void)
{
nuke *n;
if ( (n=(nuke *)malloc(sizeof(nuke))) == NULL ) exit(-1);
return n;
}

void destroyNuke(nuke *n)
{
free(n);
}

gint expandNuke(gpointer *data)
{
nuke *n;
GdkGC *gc;
GSList *nukes;
GdkColor colour;
GdkColormap *colourmap;

n = (nuke *)data;
nukes = n->list;

n->radius+=n->rd;
/*
if (n->radius>=MAXRADIUS)
	{
	n->rd=-(n->rd);
	}
*/
gc = gdk_gc_new(n->darea->window);

if (n->rd>=1)
//gdk_gc_set_foreground(gc,&(n->darea->style->black));
	{
	colour.red = (n->radius)*(65535/MAXRADIUS)*(n->red);
	colour.green = (n->radius)*(65535/MAXRADIUS)*(n->green);
	colour.blue = (n->radius)*(65535/MAXRADIUS)*(n->blue);
	colourmap = gdk_window_get_colormap(n->darea->window);
	gdk_color_alloc(colourmap,&colour);
	gdk_gc_set_foreground(gc,&colour);
	}
//else
//gdk_gc_set_foreground(gc,&(n->darea->style->white));

gdk_draw_arc(pixmap,gc,0,n->x-n->radius,n->y-n->radius,n->radius*2,n->radius*2,0,360*64);
gdk_draw_arc(n->darea->window,gc,0,n->x-n->radius,n->y-n->radius,n->radius*2,n->radius*2,0,360*64);

if (n->radius >= MAXRADIUS)
	{
	g_slist_remove(nukes,n);
 	gtk_timeout_remove(n->tickerID);
	//fprintf(stderr,"\n%d",g_slist_length(nukes)); 
	destroyNuke(n);
	}

gdk_gc_destroy(gc);
return TRUE;
}

void quit(void)
{
exit(-1);
}

GtkWidget *createDrawWindow(int xsize,int ysize,int xoff,int yoff)
{
GtkWidget *window;  
window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
gtk_window_set_policy((GtkWindow *)window,0,0,1);
gtk_widget_set_uposition(window,xoff,yoff);
gtk_window_set_title((GtkWindow *)window,"Droplet");
gtk_signal_connect(GTK_OBJECT(window),"destroy",GTK_SIGNAL_FUNC (quit), NULL);
return window;
} // end create window to draw in.

GtkWidget *createDrawArea(GtkWidget *window,GtkWidget *vbox,int xsize,int ysize)
{
GtkWidget *drawing_area;
drawing_area = gtk_drawing_area_new ();

gtk_object_set_data(GTK_OBJECT(drawing_area),"window",window);

gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), xsize, ysize);
gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);
// Signals used to handle backing pixmap
gtk_signal_connect(GTK_OBJECT (drawing_area), "expose_event",
                  (GtkSignalFunc) expose_event, NULL);
gtk_signal_connect(GTK_OBJECT(drawing_area),"configure_event",
                  (GtkSignalFunc) configure_event, NULL);
gtk_signal_connect(GTK_OBJECT(drawing_area),"button_release_event",
	          (GtkSignalFunc) button_release_event, NULL);
gtk_signal_connect(GTK_OBJECT(drawing_area),"motion_notify_event",
                  (GtkSignalFunc) motion_notify_event, NULL);

gtk_widget_set_events(drawing_area, GDK_EXPOSURE_MASK
                      | GDK_LEAVE_NOTIFY_MASK
                      | GDK_BUTTON_PRESS_MASK
                      | GDK_BUTTON_RELEASE_MASK
                      | GDK_POINTER_MOTION_MASK
                      | GDK_POINTER_MOTION_HINT_MASK);
gtk_widget_show(drawing_area);
return drawing_area;
} // end create drawing area to draw in.

GtkWidget *createVBox(GtkWidget *window)
{
GtkWidget *vbox;
vbox = gtk_vbox_new(FALSE, 2);
gtk_container_add(GTK_CONTAINER(window),vbox);
gtk_widget_show(vbox);
return vbox;
} // end create vbox for packing

gint expose_event (GtkWidget *widget, GdkEventExpose *event)
{
gdk_draw_pixmap(widget->window,
                widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                pixmap,
                event->area.x, event->area.y,
                event->area.x, event->area.y,
                event->area.width, event->area.height);

return FALSE;
}

gint configure_event (GtkWidget *widget, GdkEventConfigure *event)
{
pixmap = gdk_pixmap_new(widget->window,widget->allocation.width,widget->allocation.height,-1);
gdk_draw_rectangle(pixmap,widget->style->white_gc,TRUE,0,0,widget->allocation.width,widget->allocation.height);
return TRUE;
}

gint button_release_event(GtkWidget *widget, GdkEventButton *event)
{
GSList *nukes;

nukes = (GSList *)gtk_object_get_data(GTK_OBJECT(gtk_object_get_data(GTK_OBJECT(widget),"window")),"nukes");

dropNuke(nukes,event->x,event->y,widget,0,1,1);

return TRUE;
}

void dropNuke(GSList *nukes,gint x,gint y,GtkWidget *darea,int r,int g,int b)
{
nuke *n;

n = newNuke();
n->darea = darea;
n->x = x;
n->y = y;
n->radius = STARTRADIUS;
n->rd = 1; // radius is expanding
n->list = nukes;
n->red = r;
n->green = g;
n->blue = b;
//fprintf(stderr,"\n%d",g_slist_length(nukes));
n->tickerID = gtk_timeout_add(NUKEEXPANDSPEED,(GtkFunction)expandNuke,(gpointer)n);
g_slist_append(nukes,(gpointer)n);
}

gint motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
{
GSList *nukes;
GdkModifierType state;
gint x,y;

if (event->is_hint)
gdk_window_get_pointer (event->window, &x, &y, &state);
else
        {
        x = event->x;
        y = event->y;
        state = event->state;
        }
if (state & GDK_BUTTON1_MASK)
        {
	nukes = (GSList *)gtk_object_get_data(GTK_OBJECT(gtk_object_get_data(GTK_OBJECT(widget),"window")),"nukes");
	dropNuke(nukes,x,y,widget,0,1,1);
	}

return TRUE;
}

